@@ -1,31 +1,23 @@
 import os
 import io
-import copy
 from collections import OrderedDict
 
 import yaml
 import numpy as np
-
+from qtpy import QtCore
 from pyqtgraph import exporters
-try:
-    from qtpy import QtCore
-except ImportError:
-    from PyQt5 import QtCore
-
-from . import layout
-from . import util
 
+from .template import get_color_by_index
 
 ##################################################
 
-
 EXPORT_FILTERS = [
     ("scene to Portable Network Graphic (*.png)", [".png"]),
     ("scene to Scalable Vector Graphic (*.svg)", [".svg"]),
     ("scene to Portable Document Format (*.pdf)", [".pdf"]),
     ("data to Hierarchical Data Format (*.hdf5 *.h5)", [".hdf5", ".h5"]),
     ("data to MATLAB Binary Format (*.mat)", [".mat"]),
-    ("layout to YAML (*.yaml *.yml)", [".yaml", ".yml"]),
+    ("template to YAML (*.yaml *.yml)", [".yaml", ".yml"]),
 ]
 
 
@@ -61,7 +53,7 @@ def export_dialog(dialog_func, parent, path):
     path, selectedFilter = dialog_func(
         parent=parent,
         directory=path,
-        caption="Export scene, data, or layout to file",
+        caption="Export scene, data, or template to file",
         filter=_export_filter_spec(),
         initialFilter=initialFilter,
     )
@@ -224,37 +216,14 @@ DATA_EXPORT_FUNCTIONS = {
 # TEMPLATE
 
 
-def export_layout_yaml(plotdict, path, bw=False, window_title=None, time_window=None, font_size=10):
-    template = {}
-    template['black-on-white'] = bw
-    # convert to float from numpy.float64
-    template['time-window'] = list(map(float, time_window))
-    template['font-size'] = font_size
-    plots = []
-    for plot, cells in plotdict.items():
-        plot_item = {}
-        plot_item['channels'] = {}
-        for name, chan in plot.channels.items():
-            params = copy.deepcopy(chan.params)
-            params['color'] = params['color'].name()
-            plot_item['channels'][name] = params
-        tabspec = util.cells_to_tabspec(cells)
-        plot_item.update(tabspec)
-        if any(plot.vb.state['autoRange']):
-            plot_item['yrange'] = 'auto'
-        else:
-            # convert to float from numpy.float64
-            plot_item['yrange'] = list(map(float, plot.vb.viewRange()[1]))
-        plots.append(plot_item)
-    template['plots'] = plots
-    template['window-title'] = window_title
+def export_template_yaml(template, path):
     with open(path, 'w') as f:
         yaml.safe_dump(template, f, default_style=False)
 
 
 TEMPLATE_EXPORT_FUNCTIONS = {
-    '.yaml': export_layout_yaml,
-    '.yml': export_layout_yaml,
+    '.yaml': export_template_yaml,
+    '.yml': export_template_yaml,
 }
 
 
@@ -265,10 +234,7 @@ TEMPLATE_EXPORT_FUNCTIONS = {
 def matplotlib_plot(ddict, t0=0, window=None):
     import matplotlib.pyplot as plt
     for i, (chan, data) in enumerate(ddict.items()):
-        try:
-            color = layout.DEFAULT_PEN_COLORS[i]
-        except IndexError:
-            color = layout._random_color()
+        color = get_color_by_index(i)
         kwargs = {
             'color': color,
             'label': chan,
@@ -288,6 +254,7 @@ def matplotlib_plot(ddict, t0=0, window=None):
             ym = data['data'][mm][:]
             plt.fill_between(t, y, ym, color=color, alpha=0.5)
 
+    plt.grid(True)
     plt.legend()
     if window is not None:
         plt.xlim(*window)
