@@ -2,23 +2,20 @@ import copy
 import collections
 
 import numpy as np
-
+from qtpy import QtCore
+from qtpy.QtGui import QPen
+from qtpy.QtCore import Signal
 import pyqtgraph as pg
 from pyqtgraph import PlotItem, PlotDataItem, FillBetweenItem
-try:
-    from qtpy import QtCore
-    from qtpy.QtGui import QFont, QColor, QPen
-except ImportError:
-    from PyQt5 import QtCore
-    from PyQt5.QtGui import QFont, QColor, QPen
 
 from gpstime import gpstime, gpsnow
 
 from . import util
 from . import const
 from .plotMenu import NDScopePlotMenu
-from . import layout
+from . import template
 from . import legend
+from . import cursors
 
 
 ##################################################
@@ -63,18 +60,17 @@ def hold_over_nan(y):
 
 ##################################################
 
-
 class TimeStringAxis(pg.AxisItem):
     def __init__(self, orientation='bottom', **kwargs):
         super().__init__(orientation, **kwargs)
         self.t0 = 0
-        self.tick_func = self.t_to_relative
+        self.setTickStringsMode('relative')
 
     def t_to_relative(self, t):
         return str(util.TDStr(t))
 
     def t_to_gps(self, t):
-        return t + self.t0
+        return str(t + self.t0)
 
     def t_to_utc(self, t):
         gps = gpstime.fromgps(t + self.t0)
@@ -88,14 +84,15 @@ class TimeStringAxis(pg.AxisItem):
     def setTickStringsMode(self, mode):
         func_map = {
             'relative': self.t_to_relative,
-            'gps': self.t_to_gps,
-            'utc': self.t_to_utc,
-            'local': self.t_to_local,
+            'absolute GPS': self.t_to_gps,
+            'absolute UTC': self.t_to_utc,
+            'absolute local': self.t_to_local,
         }
         self.tick_func = func_map[mode]
         # this is needed to clear/update the ticks on change:
         self.picture = None
         self.update()
+        self.mode = mode
 
     ######
 
@@ -112,28 +109,33 @@ class TimeStringAxis(pg.AxisItem):
     def tickStrings(self, values, scale, spacing):
         return [self.tick_func(t) for t in values]
 
+
 ##################################################
 
 
 class NDScopePlot(PlotItem):
-    channel_added = QtCore.Signal(str)
-    channel_removed = QtCore.Signal(str)
-    new_plot_request = QtCore.Signal('PyQt_PyObject')
-    remove_plot_request = QtCore.Signal('PyQt_PyObject')
+    channel_select_request = Signal('PyQt_PyObject')
+    channel_add_complete = Signal()
+    new_plot_request = Signal('PyQt_PyObject')
+    remove_plot_request = Signal('PyQt_PyObject')
+    t0_reset = Signal(float)
+    t_cursors_enabled = Signal(bool)
+    t_cursor_moved = Signal('PyQt_PyObject')
 
-    def __init__(self, *args):
+    def __init__(self, data_store, *args, loc=None):
         """Initialize NDSCope Plot object
 
         """
-        super(NDScopePlot, self).__init__(
+        super().__init__(
             *args,
             axisItems={'bottom': TimeStringAxis()},
         )
 
+        self.data = data_store
+        self.loc = loc
+
         # dict of channel:curve
         self.channels = collections.OrderedDict()
-        # dict of channel:label
-        self.labels = {}
 
         # plot options
         # use automatic downsampling and clipping to reduce the
@@ -157,6 +159,8 @@ class NDScopePlot(PlotItem):
         self.setLimits(
             minXRange=0.0001,
         )
+        # accept drops
+        self.setAcceptDrops(True)
 
         self.ctrlMenu = None
         vb = self.getViewBox()
@@ -167,8 +171,15 @@ class NDScopePlot(PlotItem):
         # setting the font.
         self.titleLabel.opts.pop('size')
 
-        self.font = QFont()
-        self.text_color = QColor('gray')
+        self.t_cursors = None
+        self.y_cursors = None
+
+    def dropEvent(self, event):
+        data = event.mimeData()
+        if data.hasFormat('text/plain'):
+            text_list = data.text().splitlines()
+            for text in text_list:
+                self.add_channel(text)
 
     def add_channel(self, channel, **kwargs):
         """Add channel to plot
@@ -178,17 +189,30 @@ class NDScopePlot(PlotItem):
         """
         if channel in self.channels:
             return
-        if const.CHANNEL_RE.match(channel) is None:
-            # FIXME: should handle this error better in ancestors
-            raise ValueError("invalid channel name: {}".format(channel))
         cc = NDScopePlotChannel(channel, **kwargs)
+        self._add_channel_obj(cc)
+
+    def _add_channel_obj(self, cc):
+        channel = cc.channel
+        try:
+            self.data.add_channel(channel)
+        except ValueError:
+            return
         self.channels[channel] = cc
         self.addItem(cc.curves['y'])
         self.addItem(cc.curves['bad'])
-        self.labels[channel] = cc.label
+        self.enableAutoRange(axis='y')
         self._update_title_legend()
         cc.label_changed.connect(self._update_legend_item)
-        self.channel_added.emit(channel)
+        cc.label_changed.connect(self._update_title_legend)
+        self._update_legend_item(channel)
+
+    def _select_channel_menu(self):
+        self.channel_select_request.emit(self)
+
+    def _add_channel_menu(self, channel):
+        self.add_channel(channel)
+        self.channel_add_complete.emit()
 
     def remove_channel(self, channel):
         """remove channel from plot
@@ -197,12 +221,34 @@ class NDScopePlot(PlotItem):
         if channel not in self.channels:
             return
         cc = self.channels[channel]
+        self._remove_channel_obj(cc)
+
+    def _remove_channel_obj(self, cc):
+        channel = cc.channel
+        self.data.remove_channel(channel)
         self.legend.removeItem(cc.label)
         for curve in cc.curves.values():
             self.removeItem(curve)
         del self.channels[channel]
         self._update_title_legend()
-        self.channel_removed.emit(channel)
+
+    def set_channel_objs(self, chan_obj_list):
+        """set plot channels from list NDScopePlotChannel objects
+
+        Any channels currently being plotted that are not in list will
+        be removed.
+
+        """
+        set_chans = set(chan_obj_list)
+        cur_chans = set(self.channels.values())
+        added = False
+        for chan in cur_chans - set_chans:
+            self._remove_channel_obj(chan)
+        for chan in set_chans - cur_chans:
+            self._add_channel_obj(chan)
+            added = True
+        if added:
+            self.channel_add_complete.emit()
 
     def set_font(self, font):
         """Set label and axis font"""
@@ -211,25 +257,79 @@ class NDScopePlot(PlotItem):
         self.axes['left']['item'].label.setFont(font)
         self.axes['bottom']['item'].setTickFont(font)
         self.axes['left']['item'].setTickFont(font)
-        self.font = font
+        if self.t_cursors:
+            self.t_cursors.set_font(font)
+        if self.y_cursors:
+            self.y_cursors.set_font(font)
 
-    def set_text_color(self, color):
-        """Set color of label text and axis lines"""
-        self.titleLabel.setAttr('color', color.name())
+    def set_color_mode(self, mode):
+        fg = const.COLOR_MODE[mode]['fg']
+        bg = const.COLOR_MODE[mode]['bg']
+        self.titleLabel.setAttr('color', fg)
         self.titleLabel.setText(self.titleLabel.text)
-        self.legend.setTextColor(color)
-        self.axes['left']['item'].labelStyle['color'] = color.name()
-        self.axes['bottom']['item'].setPen(QPen(color))
-        self.axes['left']['item'].setPen(QPen(color))
-        self.text_color = color
+        self.legend.setTextColor(fg)
+        self.axes['left']['item'].set_color_mode(mode)
+        self.axes['bottom']['item'].set_color_mode(mode)
+        if self.t_cursors:
+            self.t_cursors.set_color_mode(mode)
+        if self.y_cursors:
+            self.y_cursors.set_color_mode(mode)
+        self._color_mode = mode
+
+    def _reset_t0(self, val):
+        self.t0_reset.emit(val)
+
+    def enable_t_cursors(self):
+        """enable T axis cursors"""
+        if self.t_cursors:
+            return
+        self.t_cursors = cursors.TCursors()
+        self.t_cursors.cursor_moved.connect(self._update_t_cursor)
+        self.addItem(self.t_cursors.T1)
+        self.addItem(self.t_cursors.T2)
+        self.addItem(self.t_cursors.diff)
+        self.t_cursors.set_color_mode(self._color_mode)
+
+    def disable_t_cursors(self):
+        """disable T axis cursors"""
+        if not self.t_cursors:
+            return
+        self.removeItem(self.t_cursors.T1)
+        self.removeItem(self.t_cursors.T2)
+        self.removeItem(self.t_cursors.diff)
+        self.t_cursors = None
+
+    def _update_t_cursors(self, enable):
+        self.t_cursors_enabled.emit(enable)
+
+    def _update_t_cursor(self, indval):
+        self.t_cursor_moved.emit(indval)
+
+    def enable_y_cursors(self):
+        """enable Y axis cursors"""
+        if self.y_cursors:
+            return
+        self.y_cursors = cursors.YCursors()
+        self.addItem(self.y_cursors.Y1)
+        self.addItem(self.y_cursors.Y2)
+        self.addItem(self.y_cursors.diff)
+        self.y_cursors.reset(self)
+        self.y_cursors.set_color_mode(self._color_mode)
+
+    def disable_y_cursors(self):
+        """disable Y axis cursors"""
+        if not self.y_cursors:
+            return
+        self.removeItem(self.y_cursors.Y1)
+        self.removeItem(self.y_cursors.Y2)
+        self.removeItem(self.y_cursors.diff)
+        self.y_cursors = None
 
     # SLOT
     def _update_legend_item(self, channel):
         cc = self.channels[channel]
-        label = cc.label
-        self.legend.removeItem(self.labels[channel])
-        self.legend.addItem(cc.curves['y'], label)
-        self.labels[channel] = label
+        self.legend.removeItem(cc.curves['y'])
+        self.legend.addItem(cc.curves['y'], cc.label)
 
     def _update_title_legend(self):
         """update plot title and legend"""
@@ -304,7 +404,7 @@ class NDScopePlot(PlotItem):
 
 
 class NDScopePlotChannel(QtCore.QObject):
-    label_changed = QtCore.Signal(str)
+    label_changed = Signal(str)
 
     def __init__(self, channel, **kwargs):
         """Initialize channel curve object
@@ -319,44 +419,25 @@ class NDScopePlotChannel(QtCore.QObject):
           http://www.pyqtgraph.org/documentation/functions.html#pyqtgraph.mkColor
 
         """
-        super(NDScopePlotChannel, self).__init__()
+        super().__init__()
 
         global CHANNEL_COLORS
 
         self.channel = channel
         self.ctype = None
         self.is_trend = None
-        self.params = dict(layout.CURVE)
-        self.params.update(kwargs)
-        self._update_transform()
-
-        if self.params.get('color'):
-            color = pg.mkColor(self.params['color'])
-        elif channel in CHANNEL_COLORS:
-            color = CHANNEL_COLORS[channel]
-        else:
-            while True:
-                color = pg.mkColor(layout.get_pen_color())
-                if color not in CHANNEL_COLORS.values():
-                    break
-        CHANNEL_COLORS[channel] = color
-        self.params['color'] = color
-
-        pen = pg.mkPen(color, width=self.params.get('width'))
-        mmc = copy.copy(color)
-        mmc.setAlpha(100)
-        mmpen = pg.mkPen(mmc, style=QtCore.Qt.DashLine)
+        self.params = dict(template.CURVE)
+        self.label = channel
 
         self.curves = {}
-        self.curves['y'] = PlotDataItem([0, 0], pen=pen, name=self.label)
-        self.curves['min'] = PlotDataItem([0, 0], pen=mmpen)
-        self.curves['max'] = PlotDataItem([0, 0], pen=mmpen)
+        self.curves['y'] = PlotDataItem([0, 0], name=self.label)
+        self.curves['min'] = PlotDataItem([0, 0])
+        self.curves['max'] = PlotDataItem([0, 0])
         # FIXME: fill is expensive, so we disable it until figure it out
         if False:
             self.curves['fill'] = FillBetweenItem(
                 self.curves['min'],
                 self.curves['max'],
-                brush=mmc
             )
         else:
             self.curves['fill'] = None
@@ -366,6 +447,87 @@ class NDScopePlotChannel(QtCore.QObject):
             pen=pg.mkPen('r', width=1, style=QtCore.Qt.DotLine)
         )
 
+        if channel in CHANNEL_COLORS:
+            color = CHANNEL_COLORS[channel]
+        else:
+            while True:
+                color = template.get_pen_color()
+                if color not in CHANNEL_COLORS.values():
+                    break
+        CHANNEL_COLORS[channel] = color
+        self.params['color'] = color
+
+        self.set_params(**kwargs)
+
+    def as_tuple(self):
+        return (('channel', self.channel),) + tuple(sorted(self.params.items()))
+
+    def __repr__(self):
+        return "<{} {} {}>".format(
+            self.__class__.__name__,
+            self.channel,
+            self.params,
+        )
+
+    # NOTE: we override the hash and eq definitions for these objects
+    # so that we can compare them in sets.  hopefully this doesn't
+    # cause any other problems
+    def __hash__(self):
+        return hash(self.as_tuple())
+
+    def __eq__(self, other):
+        return self.as_tuple() == other.as_tuple()
+
+    def _update_transform(self):
+        """update the transform function"""
+        if self.params['scale'] != 1 or self.params['offset'] != 0:
+            def transform(data):
+                return (data + self.params['offset']) * self.params['scale']
+        else:
+            def transform(data):
+                return data
+        self.transform = transform
+
+    def _update_label(self):
+        label = self.channel
+        if self.params['offset'] != 0:
+            label += ' {:+g}'.format(self.params['offset'])
+        if self.params['scale'] != 1:
+            label += ' *{:g}'.format(self.params['scale'])
+        if self.is_trend:
+            label += ' [{}]'.format(self.ctype)
+        self.label = label
+        self.label_changed.emit(self.channel)
+
+    def set_params(self, **params):
+        """set parameters for this channel
+
+        """
+        PARAMS = ['color', 'width', 'unit', 'scale', 'offset']
+
+        for param, value in params.items():
+            if param not in PARAMS:
+                raise KeyError(f"invalid parameter '{param}', options are: {PARAMS}")
+            self.params[param] = value
+
+        color = self.get_QColor()
+        pen = pg.mkPen(color, width=self.params['width'])
+        mmc = copy.copy(color)
+        mmc.setAlpha(100)
+        mmpen = pg.mkPen(mmc, style=QtCore.Qt.DashLine)
+        self.curves['y'].setPen(pen)
+        self.curves['min'].setPen(mmpen)
+        self.curves['max'].setPen(mmpen)
+        if self.curves['fill']:
+            self.curves['fill'].setBrush(mmc)
+        self._update_transform()
+        self._update_label()
+        self.redraw_data()
+
+    def get_QColor(self):
+        """Get channel color as a QColor object"""
+        return pg.mkColor(self.params['color'])
+
     def set_ctype(self, ctype):
         """Update the channel ctype
 
@@ -379,7 +541,7 @@ class NDScopePlotChannel(QtCore.QObject):
             self.is_trend = True
         else:
             self.is_trend = False
-        self.label_changed.emit(self.channel)
+        self._update_label()
         return True
 
     def get_unit(self, default):
@@ -389,28 +551,6 @@ class NDScopePlotChannel(QtCore.QObject):
             unit = default or 'counts'
         return unit
 
-    @property
-    def label(self):
-        """label for this channel"""
-        label = self.channel
-        if self.params['offset'] != 0:
-            label += ' {:+g}'.format(self.params['offset'])
-        if self.params['scale'] != 1:
-            label += ' *{:g}'.format(self.params['scale'])
-        if self.is_trend:
-            label += ' [{}]'.format(self.ctype)
-        return label
-
-    def _update_transform(self):
-        """update the transform function"""
-        if self.params['scale'] != 1 or self.params['offset'] != 0:
-            def transform(data):
-                return (data + self.params['offset']) * self.params['scale']
-        else:
-            def transform(data):
-                return data
-        self.transform = transform
-
     def clear_data(self):
         """clear data for curves
 
@@ -421,7 +561,7 @@ class NDScopePlotChannel(QtCore.QObject):
             except:
                 pass
 
-    def _set_curve_data(self, mod, y, t):
+    def _set_curve_data(self, mod, t, y):
         # FIXME: HACK: replace all +-infs with nans.  the infs
         # were causing the following exception in PlotCurveItem
         # when it tried to find the min/max of the array:
@@ -454,12 +594,47 @@ class NDScopePlotChannel(QtCore.QObject):
         else:
             y = data['mean']
             for mod in ['min', 'max']:
-                self._set_curve_data(mod, data[mod], t)
+                self._set_curve_data(mod, t, data[mod])
 
-        self._set_curve_data('y', y, t)
+        self._set_curve_data('y', t, y)
 
         self.curves['bad'].setData(
             x=t,
             y=hold_over_nan(y),
             connect="finite",
         )
+
+    def redraw_data(self):
+        """redraw all data, in case transform has changed"""
+        for mod, curve in self.curves.items():
+            if not curve:
+                continue
+            (t, y) = curve.getData()
+            self._set_curve_data(mod, t, y)
+
+##################################################
+
+# This is a monkey patch to add a set_color_mode method to the
+# pg.AxisItem class.
+
+# Copied from pyqtgraph source version 0.12.3.
+def _updateLabel(self):
+    """Internal method to update the label according to the text"""
+    self.label.setHtml(self.labelString())
+    self._adjustSize()
+    self.picture = None
+    self.update()
+
+pg.AxisItem._updateLabel = _updateLabel
+
+def _set_color_mode(self, mode):
+    # Modified from pyqtgraph source version 0.12.3 for AxisItem.setTextPen.
+    # https://pyqtgraph.readthedocs.io/en/pyqtgraph-0.12.3/_modules/pyqtgraph/graphicsItems/AxisItem.html#AxisItem.setTextPen
+    fg = const.COLOR_MODE[mode]['fg']
+    pen = QPen(fg)
+    self.picture = None
+    self._textPen = pen
+    self.labelStyle['color'] = self._textPen.color().name()
+    self._updateLabel()
+
+pg.AxisItem.set_color_mode = _set_color_mode
