@@ -619,6 +619,7 @@ class NDScope(*load_ui('scope.ui')):
         if yrange in [None, 'auto']:
             plot.enableAutoRange(axis='y')
         else:
+            plot.disableAutoRange(axis='y')
             plot.setYRange(*yrange)
 
         return plot
@@ -770,14 +771,22 @@ class NDScope(*load_ui('scope.ui')):
         """
         logger.info(f"loading template...")
 
-        # check the layout to make sure there's a (0,0) plot specified
-        for plot in template['plots']:
-            loc = (plot.get('row'), plot.get('col'))
-            if loc == (0, 0):
-                plot0 = plot
-                break
+        # If there is more than one plot then check the layout to make sure
+        # there's a (0,0) plot specified.
+        # If there is only one plot then move it to (0,0).
+        if len(template['plots']) == 1:
+            plot = template['plots'][0]
+            plot['row'] = 0
+            plot['col'] = 0
+            plot0 = plot
         else:
-            raise ValueError("Template layout must specify a plot at row=0, col=0")
+            for plot in template['plots']:
+                loc = (plot.get('row'), plot.get('col'))
+                if loc == (0, 0):
+                    plot0 = plot
+                    break
+            else:
+                raise ValueError("Template plot layout with more than one plot must specify a plot at row=0, col=0.")
 
         title = template.get('window-title')
         if title:
